# $Header:   //ARCHIVE/Archive/DB/sql/ApplyUpgrade.ksv   1.2   Feb 04 2001 19:00:42   BRENT  $

Usage()
{
  print "
USAGE:
  ApplyUpgrade.ksh options... <file>

The options are:
    -U<user>         User name for the database login (Default=\"$USER\")
    -P<password>     Password for the database login (Default=\"$PASSWORD\")
    -S<server>       Database server (Default=\"$SERVER\")
    -b<stepname>     Begin at the named step
    -e<stepname>     End at the named step (this step gets executed)
    -i               Run interactively. Prompt before each step.
    -o<stepname>     Run only the specified step.
    -t               Test the specified file but do not apply the steps.
    -v               Run in verbose mode. Output from all commands is sent
                     to the standard output. Without this option output is
                     sent only to log files in the current directory.
    <file>           Name of file containing upgrade instructions

Applies the upgrade instructions from a given file. Each line of the file
specifies one step to be run. Each line is of the form:
  <TYPE>|<SECONDS>|<STEPNAME>|<FLAGS>|<INSTRUCTIONS>
where <TYPE> is either 'CMD' or 'SQL' (no quotes).

<SECONDS> is a hint as to the number of seconds this instruction is
expected to take. It is used to compute the percentage-complete display.

<STEPNAME> is the name of step displayed when the step is being run. It is
also the argument to the -b<stepname> option.

If <TYPE> is 'CMD' then <INSTRUCTIONS> is a shell command line to be
executed.

If <TYPE> is 'SQL' then <INSTRUCTIONS> is the name of a SQL file to be
applied to the database. This file is a relative-path to the
\$INSTALLDIR/sql directory.

The following substitutions are supported in the CMD instructions:
  \$USER       - database user name
  \$PASSWORD   - database password
  \$SERVER     - name of the database server
  \$INSTALLDIR - Full path of the TBSM installation on the host

<FLAGS> is an integer value which specifies a bitmask of further options
used to interpret the instructions. These values can be OR'd together.
The currently defined flags are:
  0 - No options specified
  1 - when <TYPE>=SQL setting this option specifies that the referenced
      file is not expected to exist at the time the script is being run
      and therefore when using the -t option to test the upgrade file it
      is not considered an error.
"
}

# Applies the instructions in a CMD step
ApplyCMD()
{
  stepname="$1"
  command="$2"
  logfile="$PWD/$stepname.log"
  (
  cd $INSTALLDIR/sql
  eval $command >> "$logfile" 2>&1
  ec=$?
  if [ $ec -ne 0 ]; then
    print >&2 "STEP $stepname FAILED WITH EXIT CODE $ec"
    cat "$logfile"
    exit $ec
  fi
  )
  if [ $verbose = 1 ]; then
    cat "$logfile"
  fi
}

# Applies the instructions in a SQL step
ApplySQL()
{
  stepname="$1"
  sqlfile="$2"

  osql -b -U$USER -P$PASSWORD -S$SERVER -n -w3000 -H$stepname -i"$INSTALLDIR/sql/$sqlfile" >> "$stepname.log"
  ec=$?
  if [ $ec -ne 0 ]; then
    print >&2 "STEP $stepname FAILED WITH EXIT CODE $ec"
    cat "$stepname.log"
    exit $ec
  fi
  if [ $verbose = 1 ]; then
    cat "$stepname.log"
  fi
}

# Applies the specified step
ApplyStep()
{
  type="$1"
  stepname="$2"
  instructions="$3"

  echo "$(date '+%Y/%m/%d %H:%M:%S'): Starting step $stepname" > "$stepname.log"
  case $type in
  CMD)  ApplyCMD "$stepname" "$instructions";;
  SQL)  ApplySQL "$stepname" "$instructions";;
  *)    print >&2 "Unknown command type \"$type\""; exit 3;;
  esac
  echo "$(date '+%Y/%m/%d %H:%M:%S'): Finished step $stepname" >> "$stepname.log"
}

#
# main
#

interactive=0
missing_file=0
test=0
verbose=0

# Get Database Server, User, Password from Registry
eval $(regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("SERVER=%s\n",$3);} /User/ {printf("USER=%s\n",$3);} /Password/ {printf("PASSWORD=%s\n",$3);}')

while getopts b:e:iU:S:P:o:tvh? c; do
  case $c in
  b)  begin_step="$OPTARG";;
  e)  end_step="$OPTARG";;
  i)  interactive=1;;
  U)  USER="$OPTARG";;
  P)  PASSWORD="$OPTARG";;
  S)  SERVER="$OPTARG";;
  o)  only_step="$OPTARG";;
  t)  test=1;;
  v)  verbose=1;;
  h|?)  Usage; exit 0;;
  *)  Usage; exit 1;;
  esac
done

shift $OPTIND-1

file="$1"
if [ ! -f "$file" ]; then
  print >&2 "Cannot open upgrade file \"$file\""
  exit 2
fi

IFS='|'

typeset -i total_seconds=0

# Find the installation directory
eval $(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n InstallDirectory 2>nul:|grep InstallDirectory|awk '{gsub("[\\]", "/"); printf("INSTALLDIR=%s",$3);}')
if [ -z $INSTALLDIR ]; then
  print >&2 "Could not locate TBSM installation directory on this host."
  exit 2
fi

# Take a first pass through the file to read the total number of anticipated seconds

exec 4<$file
while read type seconds stepname flags instructions<&4; do
  case $type in
  [#]*) ;;
  "")   ;;
  *)
    if [ $test = 1 ]; then
      instructions=$(echo $instructions | sed "s/[$]USER/$USER/g")
      instructions=$(echo $instructions | sed "s/[$]PASSWORD/$PASSWORD/g")
      instructions=$(echo $instructions | sed "s/[$]SERVER/$SERVER/g")
      instructions=$(echo $instructions | sed "s?[$]INSTALLDIR?$INSTALLDIR?g")
      echo type=$type seconds=$seconds stepname=$stepname flags=$flags instructions=\"$instructions\"
    fi
    total_seconds=$[total_seconds+seconds]
    if [ "$type" = "SQL" ]; then
      # check for existence of file
      if [ ! -f "$INSTALLDIR/sql/$instructions" ]; then
        if [ $(($flags & 1)) = 0 ]; then
          missing_file=1
          print "  ERROR: Step $stepname. File $INSTALLDIR/sql/$instructions does not exist."
        fi
      fi
    fi
    ;;
  esac
done

echo "$INSTALLDIR|$file|$total_seconds" | awk -F'|' '{
  installdir=$1;
  file=$2;
  seconds=$3;
  printf("Installation Directory: %s\n", installdir);
  printf("Upgrade File:           %s\n", file);
  printf("Estimated Time:         %02d:%02d:%02d\n",
    seconds/3600, (seconds%3600)/60, seconds%60);
}'

if [ $missing_file = 1 ]; then
  exit 3
fi

if [ $test = 1 ]; then
  exit 0
fi

typeset -i percent_complete=0
typeset -i stepnum=0
typeset -i start_seconds=$SECONDS
typeset -i apply_step=1

# If a beginning step was specified then set apply_step to 0 until
# this step is read
if [ ! -z "$begin_step" ]; then
  apply_step=0
fi

exec 4<$file
while read type seconds stepname flags instructions<&4; do
  case $type in
  [#]*) ;;
  "")   ;;
  *)
    stepnum=$[stepnum+1]
    instructions=$(echo $instructions | sed "s/[$]USER/$USER/g")
    instructions=$(echo $instructions | sed "s/[$]PASSWORD/$PASSWORD/g")
    instructions=$(echo $instructions | sed "s/[$]SERVER/$SERVER/g")
    instructions=$(echo $instructions | sed "s?[$]INSTALLDIR?$INSTALLDIR?g")
    echo "$(date '+%Y/%m/%d %H:%M:%S')|${percent_complete}%|$stepnum|$stepname|$seconds" | awk -F'|' '{seconds=$5;printf("%s: %02d%%: Step # %3d: %-30.30s: Est. Time %02d:%02d:%02d\n",$1,$2,$3,$4,seconds/3600,(seconds%3600)/60,seconds%60);}'
    # Run the step
    if [ $apply_step = 0 -a "$begin_step" = "$stepname" ]; then
      apply_step=1
    fi
    if [ ! -z "$only_step" ]; then
      if [ "$only_step" = "$stepname" ]; then
        apply_step=1
      else
        apply_step=0
      fi
    fi
    if [ $apply_step = 1 ]; then
      if [ $interactive = 1 ]; then
        echo "Press ENTER or 'a' to apply, 's' to skip step, or 'q' to quit: \c"
        a=""
        read a
        if [ -z "$a" ]; then
          a=a
        fi
        case "$a" in
        [Aa]*)  ;;
        [Ss]*)  continue;;
        [Qq]*)  exit 0;;
        esac
      fi
      ApplyStep "$type" "$stepname" "$instructions"
    fi
    if [ "$end_step" = "$stepname" ]; then
      exit 0
    fi
    if [ "$only_step" = "$stepname" ]; then
      exit 0
    fi
    percent_complete=$[percent_complete+seconds*100/total_seconds]
    ;;
  esac
done
typeset -i end_seconds=$SECONDS
typeset -i actual_seconds=$[end_seconds-start_seconds]
echo "$(date '+%Y/%m/%d %H:%M:%S')|$actual_seconds" | awk -F'|' '{seconds=$2;printf("%s: Finished. Execution Time %02d:%02d:%02d\n",$1,seconds/3600,(seconds%3600)/60,seconds%60);}'
