-- $Header:   //ARCHIVE/Archive/DB/sql/DiscoveryBatch.sqv   1.2   Feb 04 2001 13:42:08   BRENT  $
--
-- DiscoveryBatch.sql
--
-- Implementation of the DiscoveryBatch table and related procedures.
-- DiscoveryBatch is used to record and track the progress of batch-oriented
-- discoveries which are usually initiated by loading a BCP format file into
-- a table.

-- *** IMPORTANT NOTE ***
-- The 'USE Object' command is intentionally left out since this module is run against multiple
-- databases (e.g. Object and RODMLoad)

---------------------------------------------------------------------------------------------------
-- DiscoveryBatchPStatVal defines the valid values for the DiscoveryBatch.pstat column
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'DiscoveryBatchPStatVal' AND type = 'U')
BEGIN
  PRINT 'Creating table DiscoveryBatchPStatVal'
  EXEC("
  CREATE TABLE DiscoveryBatchPStatVal
  (
    pstat   INT              CONSTRAINT DBPS_PK PRIMARY KEY(pstat),
    name    VARCHAR(30)      CONSTRAINT DBPS_name_UQ UNIQUE(name)
  )
  ")
END
GO

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = 0 AND name = 'ENQUEUED')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( 0, 'ENQUEUED' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = 1 AND name = 'LOADED')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( 1, 'LOADED' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = 2 AND name = 'INPROGRESS')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( 2, 'INPROGRESS' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = 3 AND name = 'COMPLETED')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( 3, 'COMPLETED' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = 4 AND name = 'ALLOCATED')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( 4, 'ALLOCATED' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = -1 AND name = 'LOAD_ERROR')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( -1, 'LOAD_ERROR' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = -2 AND name = 'PROCESS_ERROR')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( -2, 'PROCESS_ERROR' )

IF NOT EXISTS (SELECT * FROM DiscoveryBatchPStatVal WHERE pstat = -4 AND name = 'ALLOCATE_ERROR')
  INSERT DiscoveryBatchPStatVal(pstat, name) VALUES( -4, 'ALLOCATE_ERROR' )
GO


---------------------------------------------------------------------------------------------------
-- Drop the old DiscoveryBatch table if it exists
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'DiscoveryBatch' AND type = 'U')
  AND NOT EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('DiscoveryBatch') AND name = 'FilePath')
BEGIN
  IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CPSMDiscovery' AND type = 'U')
    EXEC("
    DROP TABLE CPSMDiscovery
    ")
  EXEC("
  DROP TABLE DiscoveryBatch
  ")
END
GO


---------------------------------------------------------------------------------------------------
-- DiscoveryBatch is used to identify and track each discovery batch as it is processed.
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'DiscoveryBatch' AND type = 'U')
BEGIN
  PRINT 'Creating table DiscoveryBatch'
  EXEC("
  CREATE TABLE DiscoveryBatch
  (
    batchid             INT IDENTITY(1,1)  CONSTRAINT DiscoveryBatch_PK PRIMARY KEY(batchid),
    ctime               DATETIME           CONSTRAINT DiscoveryBatch_ctime_DF DEFAULT GETDATE(),
    FormatType          INT,
    ActionType          INT,
    FilePath            NVARCHAR(500) NULL CONSTRAINT DiscoveryBatch_FilePath_DF DEFAULT NULL,
    CodePage            VARCHAR(8)         CONSTRAINT DiscoveryBatch_CodePage_DF DEFAULT 'ACP',
    numrows             INT NULL           CONSTRAINT DiscoveryBatch_numrows_DF DEFAULT NULL,
    pstart              DATETIME NULL      CONSTRAINT DiscoveryBatch_pstart_DF DEFAULT NULL,
    pstep               INT                CONSTRAINT DiscoveryBatch_pstep_DF DEFAULT 0,
    plast               DATETIME           CONSTRAINT DiscoveryBatch_plast_DF DEFAULT GETDATE(),
    pend                DATETIME NULL      CONSTRAINT DiscoveryBatch_pend_DF DEFAULT NULL,
    pstat               INT                CONSTRAINT DiscoveryBatch_pstat_FK FOREIGN KEY (pstat) REFERENCES DiscoveryBatchPStatVal(pstat)
                                           CONSTRAINT DiscoveryBatch_pstat_DF DEFAULT 0,
    error               INT NULL           CONSTRAINT DiscoveryBatch_error_DF DEFAULT NULL,
    errtext             VARCHAR(255) NULL  CONSTRAINT DiscoveryBatch_errtext_DF DEFAULT NULL
  )
  ")
  -- If we're in the TBSM Object database then establish a foreign key to the ACTION_TYPE table
  IF DB_NAME() = 'Object'
    EXEC("
    ALTER TABLE DiscoveryBatch ADD
      CONSTRAINT DiscoveryBatch_FORMAT_TYPE_FK FOREIGN KEY (FormatType, ActionType) REFERENCES ACTION_TYPE (FORMAT_TYPE, ACTION_TYPE)
    ")
END
GO


---------------------------------------------------------------------------------------------------
-- DiscoveryBatchAudit tracks changes to the pstat of a DiscoveryBatch
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'DiscoveryBatchAudit' AND type = 'U')
BEGIN
  PRINT 'Creating table DiscoveryBatchAudit'
  EXEC("
  CREATE TABLE DiscoveryBatchAudit
  (
    batchid             INT                CONSTRAINT DiscoveryBatchAudit_PK PRIMARY KEY(batchid, rowno),
    pstep               INT,
    plast               DATETIME,
    old_pstat           INT NULL,
    new_pstat           INT,
    error               INT NULL,
    errtext             VARCHAR(255) NULL,
    rowno               INT IDENTITY
  )
  ")
END
GO


---------------------------------------------------------------------------------------------------
-- tI_DiscoveryBatch
PRINT 'Creating trigger tI_DiscoveryBatch'
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/DiscoveryBatch.sqv  $
-- $Revision:   1.2  $
-- $Date:   Feb 04 2001 13:42:08  $
-- $Author:   BRENT  $
CREATE TRIGGER tI_DiscoveryBatch ON DiscoveryBatch FOR INSERT
AS
  INSERT DiscoveryBatchAudit(batchid, pstep, plast, old_pstat, new_pstat, error, errtext)
    SELECT i.batchid, i.pstep, i.plast, NULL, i.pstat, i.error, i.errtext
    FROM inserted AS i
GO


---------------------------------------------------------------------------------------------------
-- tU_DiscoveryBatch
PRINT 'Creating trigger tU_DiscoveryBatch'
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/DiscoveryBatch.sqv  $
-- $Revision:   1.2  $
-- $Date:   Feb 04 2001 13:42:08  $
-- $Author:   BRENT  $
CREATE TRIGGER tU_DiscoveryBatch ON DiscoveryBatch FOR UPDATE
AS
  INSERT DiscoveryBatchAudit(batchid, pstep, plast, old_pstat, new_pstat, error, errtext)
    SELECT i.batchid, i.pstep, i.plast, d.pstat, i.pstat, i.error, i.errtext
    FROM inserted AS i JOIN deleted AS d ON d.batchid = i.batchid
GO


---------------------------------------------------------------------------------------------------
-- asisp_dumpDiscoveryBatchQueue
PRINT 'Creating procedure asisp_dumpDiscoveryBatchQueue'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_dumpDiscoveryBatchQueue' AND type = 'P')
  DROP PROCEDURE asisp_dumpDiscoveryBatchQueue
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/DiscoveryBatch.sqv  $
-- $Revision:   1.2  $
-- $Date:   Feb 04 2001 13:42:08  $
-- $Author:   BRENT  $
-- Display procedure for status of DiscoveryBatch table. Output is in the standard format
-- for procedures of the form 'asisp_dump%Queue' so that asisp_dumpAllQueues will automatically
-- call this procedure.
CREATE PROCEDURE asisp_dumpDiscoveryBatchQueue
AS
  DECLARE @pending_no INT,
          @processed_no INT,
          @pending_ctime DATETIME,
          @processed_ptime DATETIME

  SELECT @pending_no=COUNT(*) FROM DiscoveryBatch WHERE pend IS NULL
  SELECT @processed_no=COUNT(*) FROM DiscoveryBatch WHERE pend IS NOT NULL
  SELECT @pending_ctime = MIN(ctime) FROM DiscoveryBatch WHERE pend IS NULL
  SELECT @processed_ptime = MAX(pend) FROM DiscoveryBatch WHERE pend IS NOT NULL

  SELECT ISNULL(@pending_no-@processed_no,0)       AS 'entries',
         ISNULL(@pending_no,0)                     AS 'pending_no',
         CONVERT(VARCHAR(30),@pending_ctime,13)    AS 'pending_ctime',
         ISNULL(@processed_no,0)                   AS 'processed_no',
         CONVERT(VARCHAR(30),@processed_ptime, 13) AS 'processed_ptime'
GO


---------------------------------------------------------------------------------------------------
-- Put an entry in the QueueThresholds table for DiscoveryBatch table.
IF NOT EXISTS (SELECT * FROM QueueThresholds WHERE queuename = 'DiscoveryBatch')
  INSERT QueueThresholds
  (
    queuename,
    entries_yellow,
    entries_red,
    last_enqueue_yellow,
    last_enqueue_red,
    last_dequeue_yellow,
    last_dequeue_red
  )
  SELECT
    'DiscoveryBatch' AS queuename,
    5                AS entries_yellow,
    10               AS entries_red,
    0                AS last_enqueue_yellow,
    0                AS last_enqueue_red,
    30               AS last_dequeue_yellow,
    60               AS last_dequeue_red
GO


---------------------------------------------------------------------------------------------------
-- _CreateDiscoveryBatch
PRINT 'Creating procedure _CreateDiscoveryBatch'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateDiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _CreateDiscoveryBatch
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/DiscoveryBatch.sqv  $
-- $Revision:   1.2  $
-- $Date:   Feb 04 2001 13:42:08  $
-- $Author:   BRENT  $
-- Creates a discovery batch with the specified FormatType, ActionType, and FileName.
-- Returns the new batch id in the @batchid output parameter.
CREATE PROCEDURE _CreateDiscoveryBatch
  @cid        ClassID,
  @id         ObjID,
  @FormatType SMALLINT,
  @ActionType SMALLINT,
  @FilePath   NVARCHAR(500),
  @CodePage   VARCHAR(8) = 'ACP',
  @batchid    INT = NULL OUTPUT
AS
  IF @FilePath IS NULL
  BEGIN
    RAISERROR('_CreateDiscoveryBatch: @FilePath cannot be NULL, aborting', 3, 1)
    RETURN -30000
  END

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _CreateDiscoveryBatch
  ELSE
    SAVE TRANSACTION _CreateDiscoveryBatch

  -- Create a discovery batch record
  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  INSERT DiscoveryBatch(ctime, FormatType, ActionType, CodePage) 
    SELECT @now, @FormatType, @ActionType, @CodePage
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _CreateDiscoveryBatch
    RETURN -30000
  END

  SELECT @batchid = batchid 
  FROM DiscoveryBatch 
  WHERE ctime = @now AND FormatType = @FormatType AND ActionType = @ActionType

  -- Compute the new file path. This is in the same directory as @FilePath, and the name is
  -- set to <FORMAT_TYPE>.<ACTION_TYPE>.<BATCHID>.BCP
  DECLARE @NewFileName NVARCHAR(80)
  SELECT @NewFileName = f.NAME + '.' + a.NAME + '.' + CAST(@batchid AS VARCHAR(9)) + '.BCP'
  FROM FORMAT_TYPE AS f, ACTION_TYPE AS a
  WHERE
    f.FORMAT_TYPE = @FormatType
    AND a.FORMAT_TYPE = @FormatType AND a.ACTION_TYPE = @ActionType

  -- Compute the full path of the new file
  DECLARE @NewFilePath NVARCHAR(500), @i INT
  SELECT @NewFilePath = REVERSE(@FilePath)
  SELECT @i = PATINDEX('%[\/]%', @NewFilePath)
  IF @i IS NOT NULL AND @i <> 0
    SELECT @NewFilePath = SUBSTRING(@NewFilePath, @i, LEN(@NewFilePath)-@i+1)
  SELECT @NewFilePath = REVERSE(@NewFilePath) + @NewFileName

  -- Update the DiscoveryBatch FilePath column with the new path
  UPDATE DiscoveryBatch SET FilePath = @NewFilePath WHERE batchid = @batchid
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _CreateDiscoveryBatch
    RETURN -30000
  END

  -- Move the specified file to the new path
  DECLARE @cmd NVARCHAR(2000), @err INT, @ret INT
  SELECT @cmd = 'MV ' + @FilePath + ' ' + @NewFilePath
  PRINT @cmd
  EXEC @ret = master..xp_cmdshell @cmd
  SELECT @err = @@ERROR
  IF @err <> 0 OR @ret <> 0
  BEGIN
    RAISERROR(N'Could not rename %s to %s (OS error %d), aborting', 3, 1, @FilePath, @NewFileName, @err)
    ROLLBACK TRANSACTION _CreateDiscoveryBatch
    RETURN -30000
  END

  IF @tcount = 0
    COMMIT TRANSACTION _CreateDiscoveryBatch
GO
